#include "dtc_float.h"
#include "math.h"

float Tem_est = 0.0;				// Estimated torque
float Phi_est = 0.0;				// Estimated flux magnitude
SpaceVector Phi_est_complex;		// Estimated flux alpha beta gamma

float theta_s = 0.0;				// Estimated flux angle in SRF

TimeDomain I_meas;					// Measured stator current abc
SpaceVector I_meas_complex;			// Measured stator current alpha beta gamma

// Switching table and vectors
TimeDomain swTable[2][3][6];
TimeDomain V0,V1,V2,V3,V4,V5,V6,V7;

bool useZSV = true;					// Use Zero Switching Vector or not
TimeDomain swVector;

// Some useful variables
PSI swIxF = PSI0;					// Index in switching table for flux
TAU swIxT = TAU0;					// Index in switching table for torque
SECTOR swIxS = S1;					// Index in switching table of sector

void RunTorqueAndFluxEstimator(float Ld, float p, float phi_alpha_r, float phi_beta_r);

SECTOR SectorFromAngle(float angle);

void initSwTable();

TAU TorqueHystComparator(float input, TAU state, float epsilon_t);

PSI FluxHystComparator(float input, PSI state, float epsilon_f);

int dtc_float(inputs_float &ins, outputs_float &outs)
{
	I_meas.A = ins.i_a;
	I_meas.B = ins.i_b;
	I_meas.C = ins.i_c;

	abc2ABG(&I_meas_complex, &I_meas);

	RunTorqueAndFluxEstimator(ins.l, ins.p, ins.phi_alpha_r, ins.phi_beta_r);

	swIxT = TorqueHystComparator(ins.Tem_ref-Tem_est, swIxT, ins.epsilon_t); // Compare estimated and reference torque

	swIxF = FluxHystComparator(ins.Phi_ref-Phi_est, swIxF, ins.epsilon_f);	// Compare estimated and reference flux

	swIxS = SectorFromAngle(theta_s);										// Find sector of the flux vector

	outs.debug_i_alpha = I_meas_complex.real;
	outs.debug_i_beta  = I_meas_complex.imaginary;
	outs.debug_psi_alpha = Phi_est_complex.real;
	outs.debug_psi_beta = Phi_est_complex.imaginary;
	outs.debug_psi_s = Phi_est;
	outs.debug_tau_s = Tem_est;
	outs.debug_theta_s = theta_s;
	outs.debug_psi_state = swIxF;
	outs.debug_sector = swIxS;
	outs.debug_Tem_state = swIxT;

	swVector = swTable[swIxF][swIxT][swIxS];							// Find appropriate switching vector from table

	return (int)swVector.A + ((int)swVector.B << 1) + ((int)swVector.C << 2);
}

void dtc_float_init()
{
	V0.A = 0; V0.B = 0; V0.C = 0;
	V1.A = 1; V1.B = 0; V1.C = 0;
	V2.A = 1; V2.B = 1; V2.C = 0;
	V3.A = 0; V3.B = 1; V3.C = 0;
	V4.A = 0; V4.B = 1; V4.C = 1;
	V5.A = 0; V5.B = 0; V5.C = 1;
	V6.A = 1; V6.B = 0; V6.C = 1;
	V7.A = 1; V7.B = 1; V7.C = 1;

	swTable[PSI1][TAU1][S1]  = V2; swTable[PSI1][TAU1][S2]  = V3; swTable[PSI1][TAU1][S3]  = V4; swTable[PSI1][TAU1][S4]  = V5; swTable[PSI1][TAU1][S5]  = V6; swTable[PSI1][TAU1][S6]  = V1;
	swTable[PSI1][TAU0][S1]  = V7; swTable[PSI1][TAU0][S2]  = V0; swTable[PSI1][TAU0][S3]  = V7; swTable[PSI1][TAU0][S4]  = V0; swTable[PSI1][TAU0][S5]  = V7; swTable[PSI1][TAU0][S6]  = V0;
	swTable[PSI1][TAU_1][S1] = V6; swTable[PSI1][TAU_1][S2] = V1; swTable[PSI1][TAU_1][S3] = V2; swTable[PSI1][TAU_1][S4] = V3; swTable[PSI1][TAU_1][S5] = V4; swTable[PSI1][TAU_1][S6] = V5;
	swTable[PSI0][TAU1][S1]  = V3; swTable[PSI0][TAU1][S2]  = V4; swTable[PSI0][TAU1][S3]  = V5; swTable[PSI0][TAU1][S4]  = V6; swTable[PSI0][TAU1][S5]  = V1; swTable[PSI0][TAU1][S6]  = V2;
	swTable[PSI0][TAU0][S1]  = V0; swTable[PSI0][TAU0][S2]  = V7; swTable[PSI0][TAU0][S3]  = V0; swTable[PSI0][TAU0][S4]  = V7; swTable[PSI0][TAU0][S5]  = V0; swTable[PSI0][TAU0][S6]  = V7;
	swTable[PSI0][TAU_1][S1] = V5; swTable[PSI0][TAU_1][S2] = V6; swTable[PSI0][TAU_1][S3] = V1; swTable[PSI0][TAU_1][S4] = V2; swTable[PSI0][TAU_1][S5] = V3; swTable[PSI0][TAU_1][S6] = V4;
}

void RunTorqueAndFluxEstimator(float Ld, float p, float phi_alpha_r, float phi_beta_r)
{
	Phi_est_complex.real = Ld*I_meas_complex.real + phi_alpha_r;
	Phi_est_complex.imaginary = Ld*I_meas_complex.imaginary + phi_beta_r;

	Phi_est = sqrt(Phi_est_complex.real*Phi_est_complex.real + Phi_est_complex.imaginary*Phi_est_complex.imaginary);

	theta_s = atan2(Phi_est_complex.imaginary, Phi_est_complex.real);

	Tem_est = 1.5*p*(Phi_est_complex.real*I_meas_complex.imaginary - Phi_est_complex.imaginary*I_meas_complex.real);
}

SECTOR SectorFromAngle(float angle)
{
	return (SECTOR)(((int)((angle+11.0*PI/6.0)*3.0/PI+1.0))%6);
}

void initSwTable()
{
	V0.A = 0; V0.B = 0; V0.C = 0;
	V1.A = 1; V1.B = 0; V1.C = 0;
	V2.A = 1; V2.B = 1; V2.C = 0;
	V3.A = 0; V3.B = 1; V3.C = 0;
	V4.A = 0; V4.B = 1; V4.C = 1;
	V5.A = 0; V5.B = 0; V5.C = 1;
	V6.A = 1; V6.B = 0; V6.C = 1;
	V7.A = 1; V7.B = 1; V7.C = 1;

	swTable[PSI1][TAU1][S1]  = V2; swTable[PSI1][TAU1][S2]  = V3; swTable[PSI1][TAU1][S3]  = V4; swTable[PSI1][TAU1][S4]  = V5; swTable[PSI1][TAU1][S5]  = V6; swTable[PSI1][TAU1][S6]  = V1;
	swTable[PSI1][TAU0][S1]  = V7; swTable[PSI1][TAU0][S2]  = V0; swTable[PSI1][TAU0][S3]  = V7; swTable[PSI1][TAU0][S4]  = V0; swTable[PSI1][TAU0][S5]  = V7; swTable[PSI1][TAU0][S6]  = V0;
	swTable[PSI1][TAU_1][S1] = V6; swTable[PSI1][TAU_1][S2] = V1; swTable[PSI1][TAU_1][S3] = V2; swTable[PSI1][TAU_1][S4] = V3; swTable[PSI1][TAU_1][S5] = V4; swTable[PSI1][TAU_1][S6] = V5;
	swTable[PSI0][TAU1][S1]  = V3; swTable[PSI0][TAU1][S2]  = V4; swTable[PSI0][TAU1][S3]  = V5; swTable[PSI0][TAU1][S4]  = V6; swTable[PSI0][TAU1][S5]  = V1; swTable[PSI0][TAU1][S6]  = V2;
	swTable[PSI0][TAU0][S1]  = V0; swTable[PSI0][TAU0][S2]  = V7; swTable[PSI0][TAU0][S3]  = V0; swTable[PSI0][TAU0][S4]  = V7; swTable[PSI0][TAU0][S5]  = V0; swTable[PSI0][TAU0][S6]  = V7;
	swTable[PSI0][TAU_1][S1] = V5; swTable[PSI0][TAU_1][S2] = V6; swTable[PSI0][TAU_1][S3] = V1; swTable[PSI0][TAU_1][S4] = V2; swTable[PSI0][TAU_1][S5] = V3; swTable[PSI0][TAU_1][S6] = V4;
}

TAU TorqueHystComparator(float input, TAU state, float epsilon_t)
{

	// Uses 3-level hysteresis comparator if zero switching vector are used
	if (useZSV) {
		if ((state == TAU1) && (input < 0))
		{
			if (input < -epsilon_t)
				return TAU_1;
			else
				return TAU0;
		}

		if (state == TAU0)
		{
			if (input < -epsilon_t)
				return TAU_1;
			if (input > epsilon_t)
				return TAU1;
		}

		if ((state == TAU_1) && (input > 0))
		{
			if (input > epsilon_t)
				return TAU1;
			else
				return TAU0;
		}

		return state;
	}
	// Uses 2-level hysteresis comparator if zero switching vector are not used
	else {
		if ((state == TAU1) && (input < -epsilon_t))
			return TAU0;
		if ((state == TAU0) && (input > epsilon_t))
			return TAU1;

		return state;
	}
}

PSI FluxHystComparator(float input, PSI state, float epsilon_f)
{
	if ((state == PSI1) && (input < -epsilon_f))
		return PSI0;
	if ((state == PSI0) && (input > epsilon_f))
		return PSI1;

	return state;
}
