
#ifndef DTC_FLOAT_H_
#define DTC_FLOAT_H_

#include "API/sensors.h"
#include "API/controllers.h"
#include "API/transformations.h"     // Coordinate transformations

#define PI 					3.141592654
#define TWOPI 				6.283185307
#define SAMPLING_FREQUENCY	(100e3)					// Sampling frequency (and main interrupt frequency)
#define SAMPLING_PERIOD		(1/SAMPLING_FREQUENCY)	// Sampling period

typedef enum { // Output of flux hysteresis comparator
	PSI0 = 0,
	PSI1 = 1
} PSI;
typedef enum { // Output of torque hysteresis comparator
	TAU_1 = 0,
	TAU0  = 1,
	TAU1  = 2
} TAU;
typedef enum { // Flux vector sector
	S1 = 0,
	S2 = 1,
	S3 = 2,
	S4 = 3,
	S5 = 4,
	S6 = 5
} SECTOR;

struct inputs_float
{

	float i_a;
	float i_b;
	float i_c;

	float l;
	float p;

	float phi_alpha_r;
	float phi_beta_r;

	float Tem_ref;
	float Phi_ref;

	float epsilon_t;
	float epsilon_f;
};

struct outputs_float
{
	float debug_psi_s;
	float debug_theta_s;
	float debug_tau_s;

	float debug_i_alpha;
	float debug_i_beta;

	float debug_psi_alpha;
	float debug_psi_beta;

	int debug_psi_state;
	int debug_sector;
	int debug_Tem_state;

	int debug_pwm_0;
	int debug_pwm_1;
	int debug_pwm_2;
};

int dtc_float(inputs_float &ins, outputs_float &outs);
void dtc_float_init();

#endif
