
#include "dtc.h"
#include "dtc_float.h"

#include "math.h"
#include <fstream>
#include <iostream>
#include <iomanip>
#include <cstdlib>

using namespace std;

#define ITERATIONS 1000

#define ADC_GAIN (10.0/32768.0)

float RunAngle(float freq, float previousAngle);

float Phi_m = 0.25;
float Phi_n = Phi_m;
float Ld = 0.0243;					// Stator d-axis inductance
float Lq = Ld;						// Stator q-axis inductance
int   p = 3;						// Pole pairs
float Tem_ref = 1.0;				// Torque reference
float Phi_ref = 0.3;				// Flux reference
float T_n = 3.9;					// Rated torque
float epsilon_t = 0.05*T_n;			// Torque hysteresis tolerance
float epsilon_f = 0.02*Phi_n;		// Flux hysteresis tolerance

float current_angle = 0.0;
float rotor_angle = 0.0;

int last_diff = 0;

int main()
{
	int retval=0;

	float I_meas_A, I_meas_B, I_meas_C;

	int errors = 0;
	int diffs = 0;

	inputs tb_ins;
	outputs tb_outs;

	inputs_float tb_ins_float;
	outputs_float tb_outs_float;

	ofstream float_result;
	ofstream dut_result;
	ofstream ins_result;
	float_result.open("dtc_float.csv");
	dut_result.open("dtc_dut.csv");
	ins_result.open("inputs.csv");

	float_result << "psi_s_f,theta_s_f,Tem_s_f,psi_state_f,Tem_state_f,sector_f,pwm0_f,pwm1_f,pwm2_f" << endl;
	dut_result << "psi_s,theta_s,Tem_s,psi_state,Tem_state,sector,pwm0,pwm1,pwm2" << endl;
	ins_result << "I_a,I_b,I_c, angle" << endl;

	dtc_float_init();

	for (int i = 0; i < ITERATIONS; i++)
	{
		current_angle = RunAngle(1000.0, current_angle);
		I_meas_A  = 5*cos(current_angle);
		I_meas_B  = 5*cos(current_angle-TWOPI/3.0);
		I_meas_C  = 5*cos(current_angle+TWOPI/3.0);

		rotor_angle = RunAngle(400.0, rotor_angle);

		//float angle = (current_angle-load_angle);
		float angle = (rotor_angle);
		float phi_alpha_r = Phi_n*cos(angle);
		float phi_beta_r = Phi_n*sin(angle);

		// FLOATING-POINT MODEL

		tb_ins_float.i_a = I_meas_A;
		tb_ins_float.i_b = I_meas_B;
		tb_ins_float.i_c = I_meas_C;

		tb_ins_float.l = Lq;
		tb_ins_float.p = p;

		tb_ins_float.phi_alpha_r = phi_alpha_r;
		tb_ins_float.phi_beta_r = phi_beta_r;

		tb_ins_float.Tem_ref = Tem_ref;
		tb_ins_float.Phi_ref = Phi_ref;

		tb_ins_float.epsilon_t = epsilon_t;
		tb_ins_float.epsilon_f = epsilon_f;

		int pwm_float = dtc_float(tb_ins_float, tb_outs_float);

		// DUT

		tb_ins.adc0 = I_meas_A / ADC_GAIN;
		tb_ins.adc1 = I_meas_B / ADC_GAIN;
		tb_ins.adc2 = I_meas_C / ADC_GAIN;

		tb_ins.adc_offset0 = 0;
		tb_ins.adc_offset1 = 0;
		tb_ins.adc_offset2 = 0;

		tb_ins.epsilon_Tem = epsilon_t / ADC_GAIN / ADC_GAIN / 4096;
		tb_ins.epsilon_phi = epsilon_f / ADC_GAIN;

		tb_ins.l = Lq * 32768;
		tb_ins.p = p;

		tb_ins.phi_ref = Phi_ref / ADC_GAIN;
		tb_ins.Tem_ref = Tem_ref / ADC_GAIN / ADC_GAIN / 4096;

		tb_ins.phi_alpha_r = phi_alpha_r / ADC_GAIN;
		tb_ins.phi_beta_r  = phi_beta_r  / ADC_GAIN;

		int pwm_dut = dtc(tb_ins,tb_outs);

		// save data in CSV files

		if(i < 1000)
		{
			ins_result << I_meas_A << "," << I_meas_B << "," << I_meas_C << ",";
			ins_result << angle << endl;

			float_result << tb_outs_float.debug_psi_s << ",";
			float_result << tb_outs_float.debug_theta_s << ",";
			float_result << tb_outs_float.debug_tau_s << ",";
			float_result << tb_outs_float.debug_psi_state << ",";
			float_result << tb_outs_float.debug_Tem_state << ",";
			float_result << tb_outs_float.debug_sector << ",";
			float_result << (pwm_float & 1) << ",";
			float_result << ((pwm_float >> 1) & 1) << ",";
			float_result << ((pwm_float >> 2) & 1) << endl;

			dut_result << tb_outs.debug_phi_s.to_float() * ADC_GAIN << ",";
			dut_result << tb_outs.debug_theta_s.to_float() / 8 << ",";
			dut_result << tb_outs.debug_Tem_s.to_float() * ADC_GAIN * ADC_GAIN * 4096 << ",";
			dut_result << tb_outs.debug_PhiHystState.to_int() << ",";
			dut_result << tb_outs.debug_TemHystState.to_int() << ",";
			dut_result << tb_outs.debug_sector.to_int() << ",";
			dut_result << (pwm_dut & 1) << ",";
			dut_result << ((pwm_dut >> 1) & 1) << ",";
			dut_result << ((pwm_dut >> 2) & 1) << endl;
		}

		if(pwm_dut != pwm_float)
		{
			diffs++;
			printf("diff #%d at i = %d\n", diffs, i);
			if(last_diff){
				errors++;
				printf("error #%d at i = %d\n", errors, i);
			}
			last_diff = 1;
		}
		else
		{
			last_diff = 0;
		}
	}

	float_result.close();
	dut_result.close();
	ins_result.close();

	if (errors > 0)
	{
		retval = 1;
		printf("------ Test failed ------\n");
	}
	else
	{
		printf("------ Test passed ------\n");
	}

	printf("Iterations: %d\n", ITERATIONS);
	printf("Differences: %d\n", diffs);
	printf("Errors: %d\n", errors);

	// Return 0 if the test passes
	return retval;
}

float RunAngle(float freq, float previousAngle)
{
	float tempAngle = previousAngle + TWOPI*freq*SAMPLING_PERIOD;

	if (tempAngle > PI) return tempAngle - TWOPI;

	return tempAngle;
}
